from django.db import models
from django.contrib.auth.models import User
from django.utils.encoding import python_2_unicode_compatible
from services.models import Service


@python_2_unicode_compatible
class WidgetData(models.Model):

    """ Metric a widget has data from
    """
    title = models.CharField(max_length=200)
    key = models.CharField(max_length=200)
    service = models.ForeignKey(Service, related_name='services_metrics',
                                null=True, blank=True)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    created_by = models.ForeignKey(User, related_name='widgetdata_created',
                                   null=True, blank=True)
    updated_by = models.ForeignKey(User, related_name='widgetdata_updated',
                                   null=True, blank=True)
    user = property(lambda self: self.created_by)

    class Meta:
        verbose_name = 'widget data'
        verbose_name_plural = 'widget data sources'

    def __str__(self):  # __unicode__ on Python 2
        return "%s (%s)" % (self.title, self.key)


@python_2_unicode_compatible
class Widget(models.Model):

    """ Metric a widget has data from
    """
    WIDGET_TYPES = (
        ('bars', 'Bar Graph'),
        ('last', 'Last'),
        ('lines', 'Lines'),
        ('pie', 'Pie Chart'),
    )
    SHOW_NULLS = (
        ('omit', 'Omit'),
        ('zeroize', 'Zeroize'),
    )
    title = models.CharField(max_length=200)
    type_of = models.CharField(max_length=10, choices=WIDGET_TYPES)
    data_from = models.CharField(max_length=20)
    interval = models.CharField(max_length=20)
    nulls = models.CharField(max_length=20, choices=SHOW_NULLS,
                             null=False, blank=False, default='zeroize',
                             help_text="What to do with intervals that have"
                             " no data.<br/>'Omit' is advised for pie charts, "
                             "'Zeroize' is advised for all other widget types")
    data = models.ManyToManyField(
        WidgetData, blank=True, help_text="Data sources to pull from.<br/>"
        "'Last' metrics are advised for 'last' widgets. 'Sum' metrics are "
        "advised for all other widget types.<br/>")
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    created_by = models.ForeignKey(User, related_name='widget_created',
                                   null=True, blank=True)
    updated_by = models.ForeignKey(User, related_name='widget_updated',
                                   null=True, blank=True)
    user = property(lambda self: self.created_by)

    class Meta:
        verbose_name = 'widget'
        verbose_name_plural = 'widgets'

    def __str__(self):  # __unicode__ on Python 2
        return "%s (%s)" % (self.title, self.type_of)


@python_2_unicode_compatible
class Dashboard(models.Model):

    """ Base Dashboard config that Widgets are collected into
    """
    name = models.CharField(max_length=200)
    widgets = models.ManyToManyField(Widget, blank=True,)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    created_by = models.ForeignKey(User, related_name='dashboard_created',
                                   null=True, blank=True)
    updated_by = models.ForeignKey(User, related_name='dashboard_updated',
                                   null=True, blank=True)
    user = property(lambda self: self.created_by)

    def __str__(self):  # __unicode__ on Python 2
        return "%s" % self.name

    class Meta:
        verbose_name = 'dashboard'
        verbose_name_plural = 'dashboards'


@python_2_unicode_compatible
class UserDashboard(models.Model):

    """ Dashboards a user is interested in
    """
    # the seed auth api user this dashboard is a profile for
    user_id = models.IntegerField(null=False, blank=False)
    dashboards = models.ManyToManyField(Dashboard, related_name='dashboards')
    default_dashboard = models.ForeignKey(Dashboard, related_name='default')
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    created_by = models.ForeignKey(User, related_name='userdashboard_created',
                                   null=True, blank=True)
    updated_by = models.ForeignKey(User, related_name='userdashboard_updated',
                                   null=True, blank=True)
    user = property(lambda self: self.created_by)

    class Meta:
        verbose_name = 'user dashboard'
        verbose_name_plural = 'users dashboards'

    def __str__(self):  # __unicode__ on Python 2
        return "Dashboards for %s" % (self.user_id, )


@python_2_unicode_compatible
class Definition(models.Model):

    """ Definitions page
    """
    title = models.CharField(max_length=200)
    description = models.CharField(max_length=200)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    created_by = models.ForeignKey(User, related_name='definition_created',
                                   null=True, blank=True)
    updated_by = models.ForeignKey(User, related_name='definition_updated',
                                   null=True, blank=True)
    user = property(lambda self: self.created_by)

    def __str__(self):  # __unicode__ on Python 2
        return "%s" % self.title



# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-09-27 16:12
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dashboards', '0002_auto_20160922_1634'),
    ]

    operations = [
        migrations.AlterField(
            model_name='widget',
            name='data',
            field=models.ManyToManyField(blank=True, help_text=b"Data sources to pull from.<br/>'Last' metrics are advised for 'last' widgets. 'Sum' metrics are advised for all other widget types.<br/>", to='dashboards.WidgetData'),
        ),
        migrations.AlterField(
            model_name='widget',
            name='nulls',
            field=models.CharField(choices=[(b'omit', b'Omit'), (b'zeroize', b'Zeroize')], default=b'zeroize', help_text=b"What to do with intervals that have no data.<br/>'Omit' is advised for pie charts, 'Zeroize' is advised for all other widget types", max_length=20),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-11-17 09:34
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('dashboards', '0003_auto_20160927_1612'),
    ]

    operations = [
        migrations.CreateModel(
            name='Definition',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=200)),
                ('description', models.CharField(max_length=200)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='definition_created', to=settings.AUTH_USER_MODEL)),
                ('updated_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='definition_updated', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'definition',
                'verbose_name_plural': 'definitions',
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-08-18 07:50
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('services', '0003_auto_20160813_1717'),
    ]

    operations = [
        migrations.AlterField(
            model_name='userservicetoken',
            name='token',
            field=models.CharField(max_length=40),
        ),
    ]



from .models import (UserDashboard, Dashboard,
                     Widget, WidgetData, Definition)
from rest_framework import serializers


class WidgetDataSerializer(serializers.ModelSerializer):
    service = serializers.StringRelatedField(many=False)

    class Meta:
        model = WidgetData
        fields = ('id', 'title', 'key', 'service')


class WidgetSerializer(serializers.ModelSerializer):
    data = WidgetDataSerializer(many=True, read_only=True)

    class Meta:
        model = Widget
        fields = ('id', 'title', 'type_of', 'data_from', 'interval',
                  'nulls', 'data')


class DashboardSerializer(serializers.ModelSerializer):
    widgets = WidgetSerializer(many=True, read_only=True)

    class Meta:
        model = Dashboard
        fields = ('id', 'name', 'widgets')


class DashboardMinimalSerializer(serializers.ModelSerializer):

    class Meta:
        model = Dashboard
        fields = ('id', 'name')


class UserDashboardSerializer(serializers.ModelSerializer):
    dashboards = DashboardMinimalSerializer(
        many=True, read_only=True)
    default_dashboard = DashboardMinimalSerializer(
        many=False, read_only=True)

    class Meta:
        model = UserDashboard
        fields = ('user_id', 'dashboards', 'default_dashboard')


class DefinitionSerializer(serializers.ModelSerializer):

    class Meta:
        model = Definition
        fields = ('id', 'title', 'description')



from .models import UserDashboard, Dashboard, Definition
from rest_framework import filters, viewsets, mixins
from rest_framework.viewsets import GenericViewSet
from rest_framework.permissions import IsAuthenticated
from .serializers import (UserDashboardSerializer, DashboardSerializer,
                          DefinitionSerializer)


class DashboardViewSet(viewsets.ReadOnlyModelViewSet):

    """
    API endpoint that allows dashboard to be viewed
    """
    permission_classes = (IsAuthenticated,)
    queryset = Dashboard.objects.all()
    serializer_class = DashboardSerializer


class UserDashboardViewSet(viewsets.ReadOnlyModelViewSet):

    """
    API endpoint that allows user dashboard to be viewed
    """
    permission_classes = (IsAuthenticated,)
    queryset = UserDashboard.objects.all()
    serializer_class = UserDashboardSerializer
    filter_backends = (filters.DjangoFilterBackend, )
    filter_fields = ('user_id', )


class DefinitionViewSet(mixins.CreateModelMixin,
                        mixins.RetrieveModelMixin,
                        mixins.UpdateModelMixin,
                        mixins.DestroyModelMixin,
                        mixins.ListModelMixin,
                        GenericViewSet):

    """
    API endpoint that allows definition to be created
    retrieved, updated and deleted
    """
    permission_classes = (IsAuthenticated,)
    queryset = Definition.objects.all()
    serializer_class = DefinitionSerializer



from django.contrib.auth.models import User, Group
from .models import Service, Status, UserServiceToken
from .tasks import get_user_token
from rest_hooks.models import Hook
from rest_framework import filters, status, viewsets
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework.permissions import IsAuthenticated, IsAdminUser
from rest_framework.pagination import LimitOffsetPagination
from .serializers import (UserSerializer, GroupSerializer,
                          ServiceSerializer, StatusSerializer, HookSerializer,
                          UserServiceTokenSerializer,
                          UserTokenRequestSerializer)


class HookViewSet(viewsets.ModelViewSet):
    """
    Retrieve, create, update or destroy webhooks.
    """
    permission_classes = (IsAuthenticated,)
    queryset = Hook.objects.all()
    serializer_class = HookSerializer

    def perform_create(self, serializer):
        serializer.save(user=self.request.user)


class UserViewSet(viewsets.ReadOnlyModelViewSet):

    """
    API endpoint that allows users to be viewed or edited.
    """
    permission_classes = (IsAdminUser,)
    queryset = User.objects.all()
    serializer_class = UserSerializer


class GroupViewSet(viewsets.ReadOnlyModelViewSet):

    """
    API endpoint that allows groups to be viewed or edited.
    """
    permission_classes = (IsAdminUser,)
    queryset = Group.objects.all()
    serializer_class = GroupSerializer


class ServiceViewSet(viewsets.ModelViewSet):

    """
    API endpoint that allows dummy models to be viewed or edited.
    """
    permission_classes = (IsAuthenticated,)
    queryset = Service.objects.all()
    serializer_class = ServiceSerializer

    def perform_create(self, serializer):
        serializer.save(created_by=self.request.user,
                        updated_by=self.request.user)

    def perform_update(self, serializer):
        serializer.save(updated_by=self.request.user)


class StatusPaginator(LimitOffsetPagination):
    default_limit = 60
    max_limit = 60


class StatusViewSet(viewsets.ReadOnlyModelViewSet):

    """
    API endpoint that allows dummy models to be viewed or edited.
    """
    permission_classes = (IsAuthenticated,)
    queryset = Status.objects.all()
    serializer_class = StatusSerializer
    filter_backends = (filters.OrderingFilter, filters.DjangoFilterBackend)
    filter_fields = ('service', 'up',)
    ordering_fields = ('created_at',)
    pagination_class = StatusPaginator

    def perform_create(self, serializer):
        serializer.save(created_by=self.request.user,
                        updated_by=self.request.user)

    def perform_update(self, serializer):
        serializer.save(updated_by=self.request.user)


class UserServiceTokenViewSet(viewsets.ReadOnlyModelViewSet):

    """
    API endpoint that allows user service tokens to be viewed
    """
    permission_classes = (IsAuthenticated,)
    queryset = UserServiceToken.objects.all()
    serializer_class = UserServiceTokenSerializer
    filter_backends = (filters.DjangoFilterBackend, )
    filter_fields = ('service', 'user_id', 'email', )


class UserServiceTokenTriggerView(APIView):

    """ UserServiceToken population - admin users only
        POST - starts up the task that fires all the scheduled metrics
    """
    permission_classes = (IsAdminUser,)

    def post(self, request, *args, **kwargs):
        serializer = UserTokenRequestSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        email = serializer.validated_data.get('email')
        user_id = serializer.validated_data.get('user_id')
        services = Service.objects.all()
        for service in services:
            get_user_token.apply_async(kwargs={
                "service_id": str(service.id),
                "user_id": user_id,
                "email": email
            })
        resp = {
            "user_service_token_initiated": True,
            "count": services.count()
        }
        return Response(
            status=status.HTTP_201_CREATED, data=resp)



import json
import uuid
import requests

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from celery.task import Task
from celery.utils.log import get_task_logger
from celery.exceptions import SoftTimeLimitExceeded
from django.utils import timezone

from seed_services_client.metrics import MetricsApiClient

from seed_control_interface_service import utils
from .models import Service, Status, UserServiceToken
from dashboards.models import WidgetData


logger = get_task_logger(__name__)


class DeliverHook(Task):
    def run(self, target, payload, instance_id=None, hook_id=None, **kwargs):
        """
        target:     the url to receive the payload.
        payload:    a python primitive data structure
        instance_id:   a possibly None "trigger" instance ID
        hook_id:       the ID of defining Hook object
        """
        requests.post(
            url=target,
            data=json.dumps(payload),
            headers={
                'Content-Type': 'application/json',
                'Authorization': 'Token %s' % settings.HOOK_AUTH_TOKEN
            }
        )


def deliver_hook_wrapper(target, payload, instance, hook):
    if instance is not None:
        if isinstance(instance.id, uuid.UUID):
            instance_id = str(instance.id)
        else:
            instance_id = instance.id
    else:
        instance_id = None
    kwargs = dict(target=target, payload=payload,
                  instance_id=instance_id, hook_id=hook.id)
    DeliverHook.apply_async(kwargs=kwargs)


def get_metric_client(session=None):
    return MetricsApiClient(
        url=settings.METRICS_URL,
        auth=(settings.METRICS_AUTH_USER, settings.METRICS_AUTH_PASSWORD),
        session=session)


class FireMetric(Task):

    """ Fires a metric using the MetricsApiClient
    """
    name = "message_sender.tasks.fire_metric"

    def run(self, metric_name, metric_value, session=None, **kwargs):
        metric_value = float(metric_value)
        metric = {
            metric_name: metric_value
        }
        metric_client = get_metric_client(session=session)
        metric_client.fire_metrics(**metric)
        return "Fired metric <%s> with value <%s>" % (
            metric_name, metric_value)


fire_metric = FireMetric()


class QueuePollService(Task):
    def run(self):
        """
        Queues all services to be polled. Should be run via beat.
        """
        services = Service.objects.all()
        for service in services:
            poll_service.apply_async(kwargs={"service_id": str(service.id)})
        return "Queued <%s> Service(s) for Polling" % services.count()


queue_poll_service = QueuePollService()


class PollService(Task):

    """
    Task to check the status of a service
    """
    name = "services.tasks.poll_service"

    class FailedEventRequest(Exception):

        """
        The attempted task failed because of a non-200 HTTP return
        code.
        """

    def get_health(self, url, token=None):
        url = "%s/api/health/" % (url, )
        headers = {"Content-Type": "application/json"}
        if token is not None:
            headers["Authorization"] = "Token %s" % (token,)
        r = requests.get(url, headers=headers)
        return r

    def run(self, service_id, **kwargs):
        """
        Retrieve a status from remote service. Set the up/down state and log.
        """
        l = self.get_logger(**kwargs)

        l.info("Loading Service for healthcheck")
        try:
            service = Service.objects.get(id=service_id)
            l.info("Getting health for <%s>" % (service.name))
            status = self.get_health(service.url, service.token)
            try:
                result = status.json()

                if (not service.up and result['up']):
                    last_up_at = service.get_last_up_time()
                    if last_up_at:
                        downtime = timezone.now() - last_up_at
                        fire_metric.apply_async(kwargs={
                            "metric_name": 'services.downtime.%s.sum' % (
                                utils.normalise_string(service.name)),
                            "metric_value": downtime.seconds // 60
                        })

                service.up = result["up"]
                service.save()
                Status.objects.create(
                    service=service,
                    up=result["up"],
                    result=result["result"]
                )
                l.info("Service <%s> up: <%s>" % (service.name, service.up))
            except:
                # can't decode means there was not a valid response
                l.info("Failed to parse response from <%s>" % (service.name))
                Status.objects.create(
                    service=service,
                    up=False,
                    result={"error": "No parseable response from service"}
                )
                service.up = False
                service.save()
            return "Completed healthcheck for <%s>" % (service.name)
        except ObjectDoesNotExist:
            logger.error('Missing Service', exc_info=True)

        except SoftTimeLimitExceeded:
            logger.error(
                'Soft time limit exceed processing poll of service \
                 via Celery.',
                exc_info=True)


poll_service = PollService()


class GetUserToken(Task):

    """
    Task to check or set the user tokens for a service
    """
    name = "services.tasks.get_user_token"

    class FailedEventRequest(Exception):

        """
        The attempted task failed because of a non-200 HTTP return
        code.
        """

    def create_token(self, url, email, token=None):
        url = "%s/api/v1/user/token/" % (url, )
        headers = {"Content-Type": "application/json"}
        if token is not None:
            headers["Authorization"] = "Token %s" % (token,)
        data = {"email": email}
        r = requests.post(url, headers=headers, data=json.dumps(data))
        return r

    def run(self, service_id, user_id, email, **kwargs):
        """
        Create and Retrieve a token from remote service. Save to DB.
        """
        l = self.get_logger(**kwargs)

        l.info("Loading Service for token creation")
        try:
            service = Service.objects.get(id=service_id)
            l.info("Getting token for <%s> on <%s>" % (email, service.name))
            response = self.create_token(service.url, email, service.token)
            try:
                result = response.json()
                ust, created = UserServiceToken.objects.get_or_create(
                    service=service, user_id=user_id, email=email)
                ust.token = result["token"]
                ust.save()
                l.info("Token saved for <%s> on <%s>" % (email, service.name))
            except:
                # can't decode means there was not a valid response
                l.info("Failed to parse response from <%s>" % (service.name))
            return "Completed getting token for <%s>" % (email)
        except ObjectDoesNotExist:
            logger.error('Missing Service', exc_info=True)

        except SoftTimeLimitExceeded:
            logger.error(
                'Soft time limit exceed processing getting service token \
                 via Celery.',
                exc_info=True)


get_user_token = GetUserToken()


class QueueServiceMetricSync(Task):
    def run(self):
        """
        Queues all services to be polled for metrics. Should be run via beat.
        """

        services = Service.objects.all()
        for service in services:
            service_metric_sync.apply_async(
                kwargs={"service_id": str(service.id)})

            key = "services.downtime.%s.sum" % (
                utils.normalise_string(service.name))
            check = WidgetData.objects.filter(service=None, key=key)
            if not check.exists():
                WidgetData.objects.create(
                    key=key,
                    title="TEMP - Pending update"
                )

        return "Queued <%s> Service(s) for Metric Sync" % services.count()


queue_service_metric_sync = QueueServiceMetricSync()


class ServiceMetricSync(Task):

    """
    Task to get the available metrics of a service
    """
    name = "services.tasks.service_metric_sync"

    class FailedEventRequest(Exception):

        """
        The attempted task failed because of a non-200 HTTP return
        code.
        """

    def get_metrics(self, url, token=None):
        url = "%s/api/metrics/" % (url, )
        headers = {"Content-Type": "application/json"}
        if token is not None:
            headers["Authorization"] = "Token %s" % (token,)
        r = requests.get(url, headers=headers)
        return r

    def run(self, service_id, **kwargs):
        """
        Retrieve a list of metrics. Ensure they are set as metric data sources.
        """
        l = self.get_logger(**kwargs)

        l.info("Loading Service for metric sync")
        try:
            service = Service.objects.get(id=service_id)
            l.info("Getting metrics for <%s>" % (service.name))
            metrics = self.get_metrics(service.url, service.token)
            result = metrics.json()
            if "metrics_available" in result:
                for key in result["metrics_available"]:
                    check = WidgetData.objects.filter(service=service, key=key)
                    if not check.exists():
                        WidgetData.objects.create(
                            service=service,
                            key=key,
                            title="TEMP - Pending update"
                        )
                        l.info("Add WidgetData for <%s>" % (key,))
            return "Completed metric sync for <%s>" % (service.name)
        except ObjectDoesNotExist:
            logger.error('Missing Service', exc_info=True)

        except SoftTimeLimitExceeded:
            logger.error(
                'Soft time limit exceed processing pull of service metrics \
                 via Celery.',
                exc_info=True)


service_metric_sync = ServiceMetricSync()



import uuid

from django.contrib.postgres.fields import JSONField
from django.contrib.auth.models import User
from django.db import models
from django.utils.encoding import python_2_unicode_compatible


@python_2_unicode_compatible
class Service(models.Model):

    """
    Base service storage model
    """
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    name = models.CharField(max_length=50, null=False, blank=False)
    url = models.CharField(max_length=255, null=False, blank=False)
    token = models.CharField(max_length=40, null=False, blank=False)
    up = models.BooleanField(default=False)
    metadata = JSONField(null=True, blank=True)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    created_by = models.ForeignKey(User, related_name='services_created',
                                   null=True)
    updated_by = models.ForeignKey(User, related_name='services_updated',
                                   null=True)
    user = property(lambda self: self.created_by)

    def __str__(self):  # __unicode__ on Python 2
        return str(self.name)

    def get_last_up_time(self):
        last_up = self.service_statuses.filter(up=True).\
            order_by('-created_at').first()
        if last_up:
            return last_up.created_at


@python_2_unicode_compatible
class Status(models.Model):

    """
    Base Status model, holds results of service pings
    """
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    service = models.ForeignKey(Service, related_name='service_statuses')
    up = models.BooleanField(default=False)
    result = JSONField(null=True, blank=True)
    created_at = models.DateTimeField(auto_now_add=True)

    def __str__(self):  # __unicode__ on Python 2
        return str("Up status is %s at %s" % (self.up, self.created_at))


@python_2_unicode_compatible
class UserServiceToken(models.Model):

    """
    User Service Token model
    Holds seed-auth-api user tokens from seed services for CLI to use
    user_id is consistent in auth-api, email might change
    """
    service = models.ForeignKey(Service, related_name='service_users')
    user_id = models.IntegerField(null=False, blank=False)
    email = models.CharField(max_length=30, null=False, blank=False)
    token = models.CharField(max_length=40, null=False, blank=False)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class meta:
        unique_together = ['service', 'user_id']

    def __str__(self):  # __unicode__ on Python 2
        return str("%s for %s at %s" % (
            self.token, self.user_id, self.service))



from django.contrib import admin

from .models import Service, Status, UserServiceToken


class ServiceAdmin(admin.ModelAdmin):
    list_display = [
        "id", "name", "url", "up",
        "created_at", "updated_at", "created_by", "updated_by"]
    list_filter = ["name", "created_at", "updated_at"]


class StatusAdmin(admin.ModelAdmin):
    list_display = [
        "id", "service", "up", "created_at"]
    list_filter = ["up", "created_at", "service"]


class UserServiceTokenAdmin(admin.ModelAdmin):
    list_display = [
        "user_id", "email", "service", "token", "created_at", "updated_at"]
    list_filter = ["user_id", "created_at", "service"]


admin.site.register(UserServiceToken, UserServiceTokenAdmin)
admin.site.register(Service, ServiceAdmin)
admin.site.register(Status, StatusAdmin)



from django.contrib.auth.models import User, Group
from .models import Service, Status, UserServiceToken
from rest_hooks.models import Hook
from rest_framework import serializers


class UserSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = ('id', 'username', 'email', 'groups')


class GroupSerializer(serializers.ModelSerializer):
    class Meta:
        model = Group
        fields = ('id', 'name')


class ServiceSerializer(serializers.ModelSerializer):

    class Meta:
        model = Service
        read_only_fields = ('created_at', 'updated_at')
        fields = ('id', 'name', 'url', 'token', 'up', 'metadata',
                  'created_at', 'created_by', 'updated_at', 'updated_by')


class ServiceLimitedSerializer(serializers.ModelSerializer):

    class Meta:
        model = Service
        read_only_fields = ('name', 'url')
        fields = ('name', 'url')


class StatusSerializer(serializers.ModelSerializer):
    service = serializers.StringRelatedField(many=False)

    class Meta:
        model = Status
        read_only_fields = ('service', 'up', 'created_at', 'result')
        fields = ('id', 'service', 'up', 'created_at', 'result')


class UserServiceTokenSerializer(serializers.ModelSerializer):
    service = ServiceLimitedSerializer(many=False)

    class Meta:
        model = UserServiceToken
        fields = ('user_id', 'email', 'service', 'token',
                  'created_at', 'updated_at')


class UserTokenRequestSerializer(serializers.Serializer):
    user_id = serializers.IntegerField()
    email = serializers.EmailField()


class HookSerializer(serializers.ModelSerializer):

    class Meta:
        model = Hook
        read_only_fields = ('user',)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-08-14 09:02
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('services', '0003_auto_20160813_1717'),
    ]

    operations = [
        migrations.CreateModel(
            name='Dashboard',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='dashboard_created', to=settings.AUTH_USER_MODEL)),
                ('updated_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='dashboard_updated', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name_plural': 'dashboards',
                'verbose_name': 'dashboard',
            },
        ),
        migrations.CreateModel(
            name='UserDashboard',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('user_id', models.IntegerField()),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='userdashboard_created', to=settings.AUTH_USER_MODEL)),
                ('dashboards', models.ManyToManyField(related_name='dashboards', to='dashboards.Dashboard')),
                ('default_dashboard', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='default', to='dashboards.Dashboard')),
                ('updated_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='userdashboard_updated', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name_plural': 'users dashboards',
                'verbose_name': 'user dashboard',
            },
        ),
        migrations.CreateModel(
            name='Widget',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=200)),
                ('type_of', models.CharField(choices=[('bars', 'Bar Graph'), ('last', 'Last'), ('lines', 'Lines'), ('pie', 'Pie Chart')], max_length=10)),
                ('data_from', models.CharField(max_length=20)),
                ('interval', models.CharField(max_length=20)),
                ('nulls', models.CharField(blank=True, choices=[('omit', 'Omit')], max_length=20, null=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='widget_created', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name_plural': 'widgets',
                'verbose_name': 'widget',
            },
        ),
        migrations.CreateModel(
            name='WidgetData',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=200)),
                ('key', models.CharField(max_length=200)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='widgetdata_created', to=settings.AUTH_USER_MODEL)),
                ('service', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='services_metrics', to='services.Service')),
                ('updated_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='widgetdata_updated', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name_plural': 'widget data sources',
                'verbose_name': 'widget data',
            },
        ),
        migrations.AddField(
            model_name='widget',
            name='data',
            field=models.ManyToManyField(blank=True, to='dashboards.WidgetData'),
        ),
        migrations.AddField(
            model_name='widget',
            name='updated_by',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='widget_updated', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='dashboard',
            name='widgets',
            field=models.ManyToManyField(blank=True, to='dashboards.Widget'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-03-08 08:31
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('dashboards', '0004_definition'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='definition',
            options={},
        ),
    ]



from django.contrib import admin
from .models import Dashboard, Widget, WidgetData, UserDashboard, Definition


admin.site.register(Dashboard)
admin.site.register(Widget)
admin.site.register(WidgetData)
admin.site.register(UserDashboard)
admin.site.register(Definition)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-09-22 16:34
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dashboards', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='widget',
            name='nulls',
            field=models.CharField(blank=True, choices=[(b'omit', b'Omit'), (b'zeroize', b'Zeroize')], max_length=20, null=True),
        ),
    ]



import re


def normalise_string(string):
    """ Strips trailing whitespace from string, lowercases it and replaces
        spaces with underscores
    """
    string = (string.strip()).lower()
    return re.sub(r'\W+', '_', string)



import json
import uuid
import responses

from datetime import datetime, timedelta

from django.contrib.auth.models import User
from django.test import TestCase
from django.conf import settings
from django.utils import timezone
from mock import patch
from rest_framework import status
from rest_framework.test import APIClient
from rest_framework.authtoken.models import Token
from seed_services_client.metrics import MetricsApiClient

from .models import Status, Service, UserServiceToken
from dashboards.models import WidgetData
from .tasks import (poll_service, get_user_token, queue_poll_service,
                    service_metric_sync, queue_service_metric_sync,
                    fire_metric)
from . import tasks


class APITestCase(TestCase):

    def setUp(self):
        self.client = APIClient()
        self.adminclient = APIClient()


class AuthenticatedAPITestCase(APITestCase):

    def _replace_get_metric_client(self, session=None):
        return MetricsApiClient(
            url=settings.METRICS_URL,
            auth=(settings.METRICS_AUTH_USER, settings.METRICS_AUTH_PASSWORD),
            session=self.session)

    def _restore_get_metric_client(self, session=None):
        return MetricsApiClient(
            url=settings.METRICS_URL,
            auth=(settings.METRICS_AUTH_USER, settings.METRICS_AUTH_PASSWORD),
            session=session)

    def setUp(self):
        super(AuthenticatedAPITestCase, self).setUp()

        tasks.get_metric_client = self._replace_get_metric_client

        self.username = 'testuser'
        self.password = 'testpass'
        self.user = User.objects.create_user(self.username,
                                             'testuser@example.com',
                                             self.password)
        token = Token.objects.create(user=self.user)
        self.token = token.key
        self.client.credentials(HTTP_AUTHORIZATION='Token ' + self.token)

        # Admin User setup
        self.adminusername = 'testadminuser'
        self.adminpassword = 'testadminpass'
        self.adminuser = User.objects.create_superuser(
            self.adminusername,
            'testadminuser@example.com',
            self.adminpassword)
        admintoken = Token.objects.create(user=self.adminuser)
        self.admintoken = admintoken.key
        self.adminclient.credentials(
            HTTP_AUTHORIZATION='Token ' + self.admintoken)
        self.primary_service = self.make_service()

    def tearDown(self):
        tasks.get_metric_client = self._restore_get_metric_client

    def make_service(self, service_data=None):
        if service_data is None:
            service_data = {
                "name": "Test Service",
                "url": "http://example.org",
                "token": str(uuid.uuid4()),
                "up": True
            }
        service = Service.objects.create(**service_data)
        return service

    def make_status(self, status_data=None):
        if status_data is None:
            status_data = {
                "service": self.primary_service,
                "up": True
            }
        status = Status.objects.create(**status_data)
        return status


class TestServicesApp(AuthenticatedAPITestCase):

    def make_user_service_token(self, user_id, service):
        user_service_token = {
            "user_id": user_id,
            "email": "%s@example.org" % user_id,
            "service": service,
            "token": str(uuid.uuid4())
        }
        token = UserServiceToken.objects.create(**user_service_token)
        return token

    def test_login(self):
        request = self.client.post(
            '/api/token-auth/',
            {"username": "testuser", "password": "testpass"})
        token = request.data.get('token', None)
        self.assertIsNotNone(
            token, "Could not receive authentication token on login post.")
        self.assertEqual(request.status_code, 200,
                         "Status code on /api/token-auth was %s -should be 200"
                         % request.status_code)

    def test_create_service_model_data(self):
        token = str(uuid.uuid4())
        post_data = {
            "name": "API Created Service",
            "url": "http://api.example.org",
            "token": token
        }
        response = self.client.post('/api/v1/service/',
                                    json.dumps(post_data),
                                    content_type='application/json')

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        d = Service.objects.get(id=response.json()["id"])
        self.assertEqual(d.name, 'API Created Service')
        self.assertEqual(d.url, 'http://api.example.org')
        self.assertEqual(d.token, token)
        self.assertEqual(d.up, False)
        self.assertEqual(d.created_by, self.user)

    def test_create_status_model_data_blocked(self):
        post_data = {
            "service": str(self.primary_service.id),
            "up": True
        }
        response = self.client.post('/api/v1/status/',
                                    json.dumps(post_data),
                                    content_type='application/json')

        self.assertEqual(response.status_code,
                         status.HTTP_405_METHOD_NOT_ALLOWED)

    def test_list_status_unfiltered(self):
        self.make_status()
        self.make_status()

        response = self.client.get('/api/v1/status/',
                                   content_type='application/json')

        self.assertEqual(response.status_code,
                         status.HTTP_200_OK)

        results = response.json()["results"]
        self.assertEqual(len(results), 2)
        self.assertEqual(results[0]["up"], True)
        self.assertEqual(results[1]["up"], True)

    def test_list_status_filtered_service(self):
        self.make_status()
        service2 = self.make_service(service_data={
            "name": "Service 2",
            "url": "http://2.example.org",
            "token": str(uuid.uuid4())
        })
        self.make_status(status_data={
            "service": service2,
            "up": False
        })

        response = self.client.get('/api/v1/status/',
                                   {"service": str(service2.id)},
                                   content_type='application/json')

        self.assertEqual(response.status_code,
                         status.HTTP_200_OK)

        results = response.json()["results"]
        self.assertEqual(Status.objects.all().count(), 2)  # two in DB
        self.assertEqual(len(results), 1)  # one in filtered response
        self.assertEqual(results[0]["up"], False)

    def test_list_status_filtered_status(self):
        self.make_status()
        service2 = self.make_service(service_data={
            "name": "Service 2",
            "url": "http://2.example.org",
            "token": str(uuid.uuid4())
        })
        self.make_status(status_data={
            "service": service2,
            "up": False
        })
        self.make_status(status_data={
            "service": service2,
            "up": True
        })

        response = self.client.get('/api/v1/status/',
                                   {"up": "True"},  # this is boolean in GET
                                   content_type='application/json')

        self.assertEqual(response.status_code,
                         status.HTTP_200_OK)

        results = response.json()["results"]
        self.assertEqual(Status.objects.all().count(), 3)  # two in DB
        self.assertEqual(len(results), 2)  # two in filtered response
        # from different services
        self.assertNotEqual(results[0]["service"], results[1]["service"])

    def test_list_user_service_token_filtered_user_id(self):
        service2 = self.make_service(service_data={
            "name": "Service 2",
            "url": "http://2.example.org",
            "token": str(uuid.uuid4())
        })
        self.make_user_service_token(user_id=1, service=self.primary_service)
        self.make_user_service_token(user_id=1, service=service2)
        self.make_user_service_token(user_id=2, service=self.primary_service)

        response = self.client.get('/api/v1/userservicetoken/',
                                   {"user_id": 1},
                                   content_type='application/json')

        self.assertEqual(response.status_code,
                         status.HTTP_200_OK)

        results = response.json()["results"]
        self.assertEqual(UserServiceToken.objects.all().count(), 3)  # 3 in DB
        self.assertEqual(len(results), 2)  # two in filtered response
        # from different services
        self.assertNotEqual(results[0]["service"], results[1]["service"])

    def test_list_user_service_token_filtered_email(self):
        service2 = self.make_service(service_data={
            "name": "Service 2",
            "url": "http://2.example.org",
            "token": str(uuid.uuid4())
        })
        self.make_user_service_token(user_id=1, service=self.primary_service)
        self.make_user_service_token(user_id=1, service=service2)
        self.make_user_service_token(user_id=2, service=self.primary_service)

        response = self.client.get('/api/v1/userservicetoken/',
                                   {"email": "2@example.org"},
                                   content_type='application/json')

        self.assertEqual(response.status_code,
                         status.HTTP_200_OK)

        results = response.json()["results"]
        self.assertEqual(UserServiceToken.objects.all().count(), 3)  # 3 in DB
        self.assertEqual(len(results), 1)  # one in filtered response
        # from different services
        self.assertEqual(results[0]["email"], "2@example.org")

    @responses.activate
    def test_task_service_poll_404(self):
        # Setup
        # mock identity lookup
        responses.add(
            responses.GET,
            'http://example.org/api/health/',
            body='<h1>404 File Not Found</h1>',
            status=404, content_type='application/json',
        )

        # Execute
        result = poll_service.apply_async(
            kwargs={
                "service_id": str(self.primary_service.id)
            })

        # Check
        self.assertEqual(
            result.get(),
            "Completed healthcheck for <Test Service>")
        updated = Service.objects.get(id=str(self.primary_service.id))
        self.assertEqual(updated.up, False)
        status = Status.objects.last()
        self.assertEqual(Status.objects.all().count(), 1)
        self.assertEqual(status.result["error"],
                         "No parseable response from service")

    @responses.activate
    def test_task_service_poll_down(self):
        # Setup
        self.session = None
        # mock identity lookup
        responses.add(
            responses.GET,
            'http://example.org/api/health/',
            json={
                "up": False,
                "result": {
                    "database": "No connection could be made"
                }
            },
            status=200, content_type='application/json',
        )

        # Execute
        result = poll_service.apply_async(
            kwargs={
                "service_id": str(self.primary_service.id)
            })

        # Check
        self.assertEqual(len(responses.calls), 1)
        self.assertEqual(
            result.get(),
            "Completed healthcheck for <Test Service>")
        updated = Service.objects.get(id=str(self.primary_service.id))
        self.assertEqual(updated.up, False)
        status = Status.objects.last()
        self.assertEqual(Status.objects.all().count(), 1)
        self.assertEqual(status.up, False)
        self.assertEqual(status.result["database"],
                         "No connection could be made")

    @responses.activate
    def test_task_service_poll_up(self):
        # Setup
        self.session = None
        dt = datetime(2017, 1, 1, tzinfo=timezone.utc)
        # mock identity lookup
        responses.add(
            responses.GET,
            'http://example.org/api/health/',
            json={
                "up": True,
                "result": {
                    "database": "Response in <0.1> seconds"
                }
            },
            status=200, content_type='application/json',
        )

        # Execute
        with patch.object(timezone, 'now', return_value=dt):
            result = poll_service.apply_async(
                kwargs={
                    "service_id": str(self.primary_service.id)
                })

        # Check
        self.assertEqual(len(responses.calls), 1)
        self.assertEqual(
            result.get(),
            "Completed healthcheck for <Test Service>")
        updated = Service.objects.get(id=str(self.primary_service.id))
        self.assertEqual(updated.up, True)
        status = Status.objects.last()
        self.assertEqual(Status.objects.all().count(), 1)
        self.assertEqual(status.up, True)
        self.assertEqual(status.result["database"],
                         "Response in <0.1> seconds")

    @responses.activate
    def test_task_queue_service_poll(self):
        # Setup
        # mock identity lookup
        responses.add(
            responses.GET,
            'http://example.org/api/health/',
            json={
                "up": True,
                "result": {
                    "database": "Response in <0.1> seconds"
                }
            },
            status=200, content_type='application/json',
        )

        # Execute
        result = queue_poll_service.apply_async()

        # Check
        self.assertEqual(
            result.get(),
            "Queued <1> Service(s) for Polling")
        updated = Service.objects.get(id=str(self.primary_service.id))
        self.assertEqual(updated.up, True)
        status = Status.objects.last()
        self.assertEqual(Status.objects.all().count(), 1)
        self.assertEqual(status.up, True)
        self.assertEqual(status.result["database"],
                         "Response in <0.1> seconds")

    @responses.activate
    def test_task_get_user_token_one_service(self):
        # Setup
        # mock identity lookup
        responses.add(
            responses.POST,
            'http://example.org/api/v1/user/token/',
            json={"token": "faketoken"},
            status=201, content_type='application/json',
        )

        # Execute
        result = get_user_token.apply_async(
            kwargs={
                "service_id": str(self.primary_service.id),
                "user_id": 5,
                "email": "test@example.org"
            })

        # Check
        self.assertEqual(
            result.get(),
            "Completed getting token for <test@example.org>")
        tokens = UserServiceToken.objects.filter(user_id=5)
        self.assertEqual(tokens.count(), 1)
        self.assertEqual(tokens[0].token, "faketoken")

    @responses.activate
    def test_task_get_user_token_two_services(self):
        # Setup
        service2 = self.make_service(service_data={
            "name": "Service 2",
            "url": "http://2.example.org",
            "token": str(uuid.uuid4())
        })
        # mock identity lookup
        responses.add(
            responses.POST,
            'http://example.org/api/v1/user/token/',
            json={"token": "faketoken"},
            status=201, content_type='application/json',
        )
        # mock other service lookup
        responses.add(
            responses.POST,
            'http://2.example.org/api/v1/user/token/',
            json={"token": "otherfaketoken"},
            status=201, content_type='application/json',
        )

        # Execute
        result = get_user_token.apply_async(
            kwargs={
                "service_id": str(self.primary_service.id),
                "user_id": 5,
                "email": "test@example.org"
            })

        result2 = get_user_token.apply_async(
            kwargs={
                "service_id": str(service2.id),
                "user_id": 5,
                "email": "test@example.org"
            })

        # Check
        self.assertEqual(
            result.get(),
            "Completed getting token for <test@example.org>")
        self.assertEqual(
            result2.get(),
            "Completed getting token for <test@example.org>")
        tokens = UserServiceToken.objects.filter(user_id=5)
        self.assertEqual(tokens.count(), 2)
        self.assertEqual(tokens[0].token, "faketoken")
        self.assertEqual(tokens[1].token, "otherfaketoken")

    @responses.activate
    def test_api_get_user_token_two_services(self):
        # Setup
        self.make_service(service_data={
            "name": "Service 2",
            "url": "http://2.example.org",
            "token": str(uuid.uuid4())
        })
        # mock identity lookup
        responses.add(
            responses.POST,
            'http://example.org/api/v1/user/token/',
            json={"token": "fakertoken"},
            status=201, content_type='application/json',
        )
        # mock other service lookup
        responses.add(
            responses.POST,
            'http://2.example.org/api/v1/user/token/',
            json={"token": "otherfakertoken"},
            status=201, content_type='application/json',
        )

        # Execute
        post_data = {
            "user_id": 5,
            "email": "test@example.org"
        }
        response = self.adminclient.post('/api/v1/userservicetoken/generate/',
                                         json.dumps(post_data),
                                         content_type='application/json')
        results = response.json()

        # Check
        self.assertEqual(response.status_code,
                         status.HTTP_201_CREATED)
        self.assertEqual(results["user_service_token_initiated"], True)
        self.assertEqual(results["count"], 2)
        tokens = UserServiceToken.objects.filter(user_id=5)
        self.assertEqual(tokens.count(), 2)
        self.assertEqual(tokens[0].token, "fakertoken")
        self.assertEqual(tokens[1].token, "otherfakertoken")

    @responses.activate
    def test_task_service_metric_sync(self):
        # Setup
        # mock metrics lookup
        responses.add(
            responses.GET,
            'http://example.org/api/metrics/',
            json={
                "metrics_available": [
                    "identities.created.sum",
                    "identities.created.last"
                ]
            },
            status=200, content_type='application/json',
        )

        # Execute
        result = service_metric_sync.apply_async(
            kwargs={
                "service_id": str(self.primary_service.id)
            })

        # Check
        self.assertEqual(
            result.get(),
            "Completed metric sync for <Test Service>")
        widget_data = WidgetData.objects.filter(service=self.primary_service)
        self.assertEqual(widget_data.count(), 2)

    @responses.activate
    def test_task_queue_metric_poll(self):
        # Setup
        # mock metrics lookup
        responses.add(
            responses.GET,
            'http://example.org/api/metrics/',
            json={
                "metrics_available": [
                    "identities.created.sum",
                    "identities.created.last"
                ]
            },
            status=200, content_type='application/json',
        )

        # Execute twice
        result = queue_service_metric_sync.apply_async()
        result2 = queue_service_metric_sync.apply_async()

        # Check - should be two, despite being run twice
        self.assertEqual(
            result.get(),
            "Queued <1> Service(s) for Metric Sync")
        self.assertEqual(
            result2.get(),
            "Queued <1> Service(s) for Metric Sync")
        widget_data = WidgetData.objects.filter(service=self.primary_service)
        self.assertEqual(widget_data.count(), 2)

        # Check that CI-Service metric is inserted
        widgets = WidgetData.objects.filter(service=None).values_list(
            'key', flat=True)
        self.assertEqual(list(widgets), ['services.downtime.test_service.sum'])


class TestMetrics(AuthenticatedAPITestCase):

    @responses.activate
    def test_direct_fire(self):
        # Setup
        self.session = None
        responses.add(responses.POST,
                      "http://metrics-url/metrics/",
                      json={"foo": "bar"},
                      status=200, content_type='application/json')
        # Execute
        result = fire_metric.apply_async(kwargs={
            "metric_name": 'foo.last',
            "metric_value": 1
        })
        # Check
        self.assertEqual(json.loads(responses.calls[0].request.body), {
            "foo.last": 1.0
        })
        self.assertEqual(result.get(),
                         "Fired metric <foo.last> with value <1.0>")

    @responses.activate
    def test_downtime_metric(self):
        # Setup
        self.session = None
        dt = datetime(2017, 1, 1, tzinfo=timezone.utc)

        self.primary_service.up = False
        self.primary_service.save()

        # last up status
        status = self.make_status()
        status.created_at = dt - timedelta(minutes=65)
        status.save()

        # up status that's a little bit older
        status = self.make_status()
        status.created_at = dt - timedelta(minutes=90)
        status.save()

        # newer down status
        status = self.make_status()
        status.created_at = dt - timedelta(minutes=20)
        status.up = False
        status.save()

        # mock health lookup
        responses.add(
            responses.GET,
            'http://example.org/api/health/',
            json={
                "up": True,
                "result": {
                    "database": "Response in <0.1> seconds"
                }
            },
            status=200, content_type='application/json',
        )

        responses.add(responses.POST,
                      "http://metrics-url/metrics/",
                      json={"foo": "bar"},
                      status=200, content_type='application/json')

        # Execute
        with patch.object(timezone, 'now', return_value=dt):
            poll_service.apply_async(
                kwargs={
                    "service_id": str(self.primary_service.id)
                })

        # Check
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(json.loads(responses.calls[1].request.body), {
            "services.downtime.test_service.sum": 65.0
        })



from django.conf.urls import url, include
from rest_framework import routers
from . import views

router = routers.DefaultRouter()
router.register(r'dashboard', views.DashboardViewSet)
router.register(r'userdashboard', views.UserDashboardViewSet)
router.register(r'definition', views.DefinitionViewSet)

# Wire up our API using automatic URL routing.
urlpatterns = [
    url(r'^api/v1/', include(router.urls)),
]



from django.conf.urls import url, include
from rest_framework import routers
from . import views

router = routers.DefaultRouter()
router.register(r'user', views.UserViewSet)
router.register(r'group', views.GroupViewSet)
router.register(r'service', views.ServiceViewSet)
router.register(r'status', views.StatusViewSet)
router.register(r'webhook', views.HookViewSet)
router.register(r'userservicetoken', views.UserServiceTokenViewSet)


# Wire up our API using automatic URL routing.
# Additionally, we include login URLs for the browseable API.
urlpatterns = [
    url(r'^api/v1/userservicetoken/generate/$',
        views.UserServiceTokenTriggerView.as_view(),
        name='generate-user-tokens'),
    url(r'^api/v1/', include(router.urls)),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-08-03 12:55
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('services', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserServiceToken',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('user_id', models.IntegerField()),
                ('email', models.CharField(max_length=30)),
                ('token', models.CharField(max_length=36)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('service', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='service_users', to='services.Service')),
            ],
        ),
    ]
